#include <string.h>  // memcpy()
#include <stdio.h>   // sscanf() etc.
#include <stdbool.h>
#include <stm32f10x_can.h>
#include <stm32f10x_rcc.h>
#include <misc.h>
#include "common_delay.h"
#include "common_LED.h"
#include "common_CAN.h"

extern uint32_t const THIS_NODE_ID;

CanRxMsg CAN_RxMessage={0};
CanTxMsg CAN_TxMessage={0};
char i = 0;

void can_tx_message_init(CanTxMsg *message)
{
  message->StdId = THIS_NODE_ID; 
  message->ExtId = 0;
  message->IDE = CAN_ID_STD;
  message->RTR = CAN_RTR_DATA;    
  message->DLC = 8;
}

uint8_t can_messages_available(void)
{
  return CAN_MessagePending(CAN1, CAN_FIFO0);
} 

uint8_t can_read_message(uint32_t *data_type, uint32_t *origin_id, uint8_t message_data[])
{
    CAN_Receive(CAN1, CAN_FIFO0, &CAN_RxMessage);
    *data_type = can_rx_message_data_type(CAN_RxMessage.StdId);    
    *origin_id = can_rx_message_origin_id(CAN_RxMessage.StdId);
  
    memcpy(message_data, CAN_RxMessage.Data, 8);
    //If you said all, that means me
    if(*data_type == ID_ALL_NODES) *data_type = THIS_NODE_ID;
  return 1;
}

uint8_t can_send_message(uint32_t data_type, uint8_t message_data[])
{
  CAN_TxMessage.StdId = (data_type<<5) + THIS_NODE_ID;
  memcpy(CAN_TxMessage.Data, message_data, 8);
  CAN_Transmit(CAN1, &CAN_TxMessage);
  return 1;
}

uint32_t can_rx_message_origin_id(uint32_t StdId)
{ //origin is low 5 bits of message.StdId
  return StdId & 0x1F; //0001 1111
}

uint32_t can_rx_message_data_type(uint32_t StdId)
{ //data_type is high 6 bits of message.StdId
  return (StdId>>5) & 0x3F; //0011 1111
}

void can_filter_n_config(uint8_t FilterNumber, int idLo, int maskLo, int idHigh, int maskHigh)
{
  //  63d is 11 1111 is 0x3F
  //  CAN_filterN_Config(1, 0<<10, 63<<10, MOTOR_CONTROL<<10, 63<<10);

  CAN_FilterInitTypeDef  CAN_FilterInitStructure;

  CAN_FilterInitStructure.CAN_FilterNumber = FilterNumber;
  CAN_FilterInitStructure.CAN_FilterMode = CAN_FilterMode_IdMask;//anything if 0
  CAN_FilterInitStructure.CAN_FilterScale = CAN_FilterScale_16bit;
  
  CAN_FilterInitStructure.CAN_FilterIdHigh = idHigh; 
  CAN_FilterInitStructure.CAN_FilterMaskIdHigh = maskHigh;
  
  CAN_FilterInitStructure.CAN_FilterIdLow  = idLo; 
  CAN_FilterInitStructure.CAN_FilterMaskIdLow  = maskLo;
  CAN_FilterInitStructure.CAN_FilterFIFOAssignment = 0;
  CAN_FilterInitStructure.CAN_FilterActivation = ENABLE;
  CAN_FilterInit(&CAN_FilterInitStructure);
}

void can_filter_data_type(uint8_t FilterNumber, int data_type1, int data_type2)
{
  //need to shift from 0000 0000 00nn nnnn 
  //to                 nnnn nn00 0000 0000  ie <<10
  //mask is            1111 1100 0000 0000  ie 0xFC00
  can_filter_n_config(FilterNumber, data_type1<<10, 0xFC00, data_type2<<10, 0xFC00);
}

void can_filter_origin(uint8_t FilterNumber, int origin1, int origin2)
{
  //need to shift from 0000 0000 000n nnnn 
  //to                 0000 00nn nnn0 0000  ie <<5
  //mask is            0000 0011 1110 0000  ie 0x03E0
  can_filter_n_config(FilterNumber, origin1<<5, 0x03E0, origin2<<5, 0x03E0);
}

void can_config(void)
{
  GPIO_InitTypeDef  GPIO_InitStructure;
  CAN_InitTypeDef   CAN_InitStructure;
  
  /* GPIO clock enable */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO | RCC_APB2Periph_GPIOA, ENABLE);
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_CAN1, ENABLE); 
  
  /* Configure CAN pin: RX */     
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING; 
  GPIO_Init(GPIOA, &GPIO_InitStructure);
  
  /* Configure CAN pin: TX */   
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP; 
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
  
/* CAN register init */
  CAN_DeInit(CAN1);
  CAN_StructInit(&CAN_InitStructure);

  CAN_InitStructure.CAN_TTCM = DISABLE;
  CAN_InitStructure.CAN_ABOM = DISABLE;
  CAN_InitStructure.CAN_AWUM = DISABLE;
  CAN_InitStructure.CAN_NART = DISABLE;
  CAN_InitStructure.CAN_RFLM = DISABLE;
  CAN_InitStructure.CAN_TXFP = DISABLE;
  CAN_InitStructure.CAN_Mode = CAN_Mode_Normal; 
    
  //(1+13+4)*4=72 
  // 1+13+4=18 1+13=14 14/18= sp77.8% tq18
  CAN_InitStructure.CAN_SJW = CAN_SJW_1tq; 
  CAN_InitStructure.CAN_BS1 = CAN_BS1_13tq;
  CAN_InitStructure.CAN_BS2 = CAN_BS2_4tq;
  CAN_InitStructure.CAN_Prescaler = 4; //500K
//CAN_InitStructure.CAN_Prescaler = 2; //1meg
  CAN_Init(CAN1, &CAN_InitStructure);

  can_filter_data_type(0, THIS_NODE_ID, ID_ALL_NODES); 
  can_tx_message_init(&CAN_TxMessage);
}

void can_nvic_config(void)
{
  NVIC_InitTypeDef  NVIC_InitStructure;  
  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_0);
  NVIC_InitStructure.NVIC_IRQChannel = USB_LP_CAN1_RX0_IRQn;
  
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);  
  CAN_ITConfig(CAN1, CAN_IT_FMP0, ENABLE);
}

void example_USB_LP_CAN1_RX0_IRQHandler(void)
{ //remove "example_"
  uint32_t data_type=0;
  if(CAN_MessagePending(CAN1, CAN_FIFO0) > 0)
  { 
    CAN_Receive(CAN1, CAN_FIFO0, &CAN_RxMessage);
    data_type = can_rx_message_data_type(CAN_RxMessage.StdId);    
    if(data_type==THIS_NODE_ID) //command message
    {  
//      if(CAN_RxMessage.Data[1]=='1')LED_on();
//      else LED_off();
    }
  }
}  

uint8_t scan_cmd_str(uint8_t command_string[], uint8_t *command_char, int16_t *value)
{
  sscanf((char*)command_string,"#%c%6d", command_char, (int*)value);
  return 1;
}

uint8_t universal_command(uint8_t command_char, int16_t value)
{
  uint8_t hit=0;
  switch(command_char)          
  {
    case '*':
      led_command(value);
    hit=1;
    break;            

    case '~':
      if(value==666) NVIC_SystemReset();
    hit=1;
    break;   
  
  }//end switch 
  return hit;
}
